@@ -32,6 +32,7 @@
 #include "gdkdrawable.h"
 #include "gdkpixmap.h"
 #include "gdkscreen.h"
+#include "gdkalias.h"
 
 #define USE_BACKING_STORE	/* Appears to work on Win32, too, now. */
 
@@ -43,6 +44,7 @@ struct _GdkWindowPaint
   GdkPixmap *pixmap;
   gint x_offset;
   gint y_offset;
+  cairo_surface_t *surface;
 };
 
 static GdkGC *gdk_window_create_gc      (GdkDrawable     *drawable,
@@ -104,12 +106,20 @@ static void   gdk_window_draw_lines     (GdkDrawable     *drawable,
 					 GdkGC           *gc,
 					 GdkPoint        *points,
 					 gint             npoints);
-static void   gdk_window_draw_glyphs    (GdkDrawable      *drawable,
-					 GdkGC            *gc,
-					 PangoFont        *font,
-					 gint              x,
-					 gint              y,
-					 PangoGlyphString *glyphs);
+
+static void gdk_window_draw_glyphs             (GdkDrawable      *drawable,
+						GdkGC            *gc,
+						PangoFont        *font,
+						gint              x,
+						gint              y,
+						PangoGlyphString *glyphs);
+static void gdk_window_draw_glyphs_transformed (GdkDrawable      *drawable,
+						GdkGC            *gc,
+						PangoMatrix      *matrix,
+						PangoFont        *font,
+						gint              x,
+						gint              y,
+						PangoGlyphString *glyphs);
 
 static void   gdk_window_draw_image     (GdkDrawable     *drawable,
                                          GdkGC           *gc,
@@ -134,6 +144,11 @@ static void gdk_window_draw_pixbuf (GdkDrawable     *drawable,
 				    gint             x_dither,
 				    gint             y_dither);
 
+static void gdk_window_draw_trapezoids (GdkDrawable   *drawable,
+					GdkGC	      *gc,
+					GdkTrapezoid  *trapezoids,
+					gint           n_trapezoids);
+
 static GdkImage* gdk_window_copy_to_image (GdkDrawable *drawable,
 					   GdkImage    *image,
 					   gint         src_x,
@@ -143,6 +158,8 @@ static GdkImage* gdk_window_copy_to_image (GdkDrawable *drawable,
 					   gint         width,
 					   gint         height);
 
+static cairo_surface_t *gdk_window_ref_cairo_surface (GdkDrawable *drawable);
+
 static void   gdk_window_real_get_size  (GdkDrawable     *drawable,
                                          gint            *width,
                                          gint            *height);
@@ -238,8 +255,10 @@ gdk_window_class_init (GdkWindowObjectClass *klass)
   drawable_class->draw_segments = gdk_window_draw_segments;
   drawable_class->draw_lines = gdk_window_draw_lines;
   drawable_class->draw_glyphs = gdk_window_draw_glyphs;
+  drawable_class->draw_glyphs_transformed = gdk_window_draw_glyphs_transformed;
   drawable_class->draw_image = gdk_window_draw_image;
   drawable_class->draw_pixbuf = gdk_window_draw_pixbuf;
+  drawable_class->draw_trapezoids = gdk_window_draw_trapezoids;
   drawable_class->get_depth = gdk_window_real_get_depth;
   drawable_class->get_screen = gdk_window_real_get_screen;
   drawable_class->get_size = gdk_window_real_get_size;
@@ -247,6 +266,7 @@ gdk_window_class_init (GdkWindowObjectClass *klass)
   drawable_class->get_colormap = gdk_window_real_get_colormap;
   drawable_class->get_visual = gdk_window_real_get_visual;
   drawable_class->_copy_to_image = gdk_window_copy_to_image;
+  drawable_class->ref_cairo_surface = gdk_window_ref_cairo_surface;
   drawable_class->get_clip_region = gdk_window_get_clip_region;
   drawable_class->get_visible_region = gdk_window_get_visible_region;
   drawable_class->get_composite_drawable = gdk_window_get_composite_drawable;
@@ -826,7 +846,7 @@ gdk_window_is_viewable (GdkWindow *window)
 	 (private != (GdkWindowObject *)root_window) &&
 	 (GDK_WINDOW_TYPE (private) != GDK_WINDOW_FOREIGN))
     {
-      if (GDK_WINDOW_DESTROYED (window) || !GDK_WINDOW_IS_MAPPED (window))
+      if (GDK_WINDOW_DESTROYED (private) || !GDK_WINDOW_IS_MAPPED (private))
 	return FALSE;
       
       private = (GdkWindowObject *)private->parent;
@@ -953,6 +973,10 @@ gdk_window_begin_paint_region (GdkWindow *window,
     gdk_pixmap_new (window,
 		    MAX (clip_box.width, 1), MAX (clip_box.height, 1), -1);
 
+  paint->surface = _gdk_drawable_ref_cairo_surface (paint->pixmap);
+  cairo_surface_set_device_offset (paint->surface,
+				   - paint->x_offset, - paint->y_offset);
+  
   for (list = private->paint_stack; list != NULL; list = list->next)
     {
       GdkWindowPaint *tmp_paint = list->data;
@@ -1027,7 +1051,8 @@ gdk_window_end_paint (GdkWindow *window)
 
   /* Reset clip region of the cached GdkGC */
   gdk_gc_set_clip_region (tmp_gc, NULL);
-  
+
+  cairo_surface_destroy (paint->surface);
   g_object_unref (paint->pixmap);
   gdk_region_destroy (paint->region);
   g_free (paint);
@@ -1655,41 +1680,88 @@ gdk_window_draw_glyphs (GdkDrawable      *drawable,
   RESTORE_GC (gc);
 }
 
-static GdkGC *
-gdk_window_get_bg_gc (GdkWindow      *window,
-		      GdkWindowPaint *paint)
+static void
+gdk_window_draw_glyphs_transformed (GdkDrawable      *drawable,
+				    GdkGC            *gc,
+				    PangoMatrix      *matrix,
+				    PangoFont        *font,
+				    gint              x,
+				    gint              y,
+				    PangoGlyphString *glyphs)
+{
+  GdkWindowObject *private = (GdkWindowObject *)drawable;
+  PangoMatrix tmp_matrix;
+
+  OFFSET_GC (gc);
+
+  if (GDK_WINDOW_DESTROYED (drawable))
+    return;
+
+  if (x_offset != 0 || y_offset != 0)
+    {
+      if (matrix)
+	{
+	  tmp_matrix = *matrix;
+	  tmp_matrix.x0 -= x_offset;
+	  tmp_matrix.y0 -= y_offset;
+	  matrix = &tmp_matrix;
+	}
+      else
+	{
+	  x -= x_offset * PANGO_SCALE;
+	  y -= y_offset * PANGO_SCALE;
+	}
+    }
+  
+  if (private->paint_stack)
+    {
+      GdkWindowPaint *paint = private->paint_stack->data;
+
+      gdk_draw_glyphs_transformed (paint->pixmap, gc, matrix, font, x, y, glyphs);
+    }
+  else
+    gdk_draw_glyphs_transformed (private->impl, gc, matrix, font, x, y, glyphs);
+
+  RESTORE_GC (gc);
+}
+
+static void
+gdk_window_set_bg_pattern (GdkWindow      *window,
+			   cairo_t        *cr,
+			   int             x_offset,
+			   int             y_offset)
 {
   GdkWindowObject *private = (GdkWindowObject *)window;
 
-  guint gc_mask = 0;
-  GdkGCValues gc_values;
-
   if (private->bg_pixmap == GDK_PARENT_RELATIVE_BG && private->parent)
     {
-      GdkWindowPaint tmp_paint = *paint;
-      tmp_paint.x_offset += private->x;
-      tmp_paint.y_offset += private->y;
-      
-      return gdk_window_get_bg_gc (GDK_WINDOW (private->parent), &tmp_paint);
+      x_offset += private->x;
+      y_offset += private->y;
+      gdk_window_set_bg_pattern (GDK_WINDOW (private->parent), cr,
+				 x_offset, y_offset);
     }
   else if (private->bg_pixmap && 
            private->bg_pixmap != GDK_PARENT_RELATIVE_BG && 
            private->bg_pixmap != GDK_NO_BG)
     {
-      gc_values.fill = GDK_TILED;
-      gc_values.tile = private->bg_pixmap;
-      
-      gc_mask = GDK_GC_FILL | GDK_GC_TILE;
+      cairo_surface_t *surface = _gdk_drawable_ref_cairo_surface (private->bg_pixmap);
+      cairo_pattern_t *pattern = cairo_pattern_create_for_surface (surface);
+      cairo_surface_destroy (surface);
 
-      return gdk_gc_new_with_values (paint->pixmap, &gc_values, gc_mask);
+      if (x_offset != 0 || y_offset != 0)
+	{
+	  cairo_matrix_t matrix;
+	  cairo_matrix_init_translate (&matrix, x_offset, y_offset);
+	  cairo_pattern_set_matrix (pattern, &matrix);
+	}
+
+      cairo_pattern_set_extend (pattern, CAIRO_EXTEND_REPEAT);
+      cairo_set_source (cr, pattern);
+      cairo_pattern_destroy (pattern);
     }
   else
     {
-      GdkGC *gc = _gdk_drawable_get_scratch_gc (paint->pixmap, FALSE);
-
-      gdk_gc_set_foreground (gc, &(private->bg_color));
-
-      return g_object_ref (gc);
+      gdk_cairo_set_source_color (cr, &private->bg_color);
     }
 }
 
@@ -1702,20 +1774,22 @@ gdk_window_clear_backing_rect (GdkWindow *window,
 {
   GdkWindowObject *private = (GdkWindowObject *)window;
   GdkWindowPaint *paint = private->paint_stack->data;
-  GdkGC *tmp_gc;
+  cairo_t *cr;
 
   if (GDK_WINDOW_DESTROYED (window))
     return;
 
-  tmp_gc = gdk_window_get_bg_gc (window, paint);
-  gdk_gc_set_clip_region (tmp_gc, paint->region);
-  
-  gdk_draw_rectangle (window, tmp_gc, TRUE,
-		      x, y, width, height);
+  cr = cairo_create (paint->surface);
 
-  gdk_gc_set_clip_region (tmp_gc, NULL);
-  
-  g_object_unref (tmp_gc);
+  gdk_window_set_bg_pattern (window, cr, 0, 0);
+
+  cairo_rectangle (cr, x, y, width, height);
+  cairo_clip (cr);
+
+  gdk_cairo_region (cr, paint->region);
+  cairo_fill (cr);
+
+  cairo_destroy (cr);
 }
 
 /**
@@ -1894,6 +1968,52 @@ gdk_window_draw_pixbuf (GdkDrawable     *drawable,
     }
 }
 
+static void
+gdk_window_draw_trapezoids (GdkDrawable   *drawable,
+			    GdkGC	  *gc,
+			    GdkTrapezoid  *trapezoids,
+			    gint           n_trapezoids)
+{
+  GdkWindowObject *private = (GdkWindowObject *)drawable;
+  GdkTrapezoid *new_trapezoids = NULL;
+
+  OFFSET_GC (gc);
+
+  if (GDK_WINDOW_DESTROYED (drawable))
+    return;
+  
+  if (x_offset != 0 || y_offset != 0)
+    {
+      gint i;
+
+      new_trapezoids = g_new (GdkTrapezoid, n_trapezoids);
+      for (i=0; i < n_trapezoids; i++)
+	{
+	  new_trapezoids[i].y1 = trapezoids[i].y1 - y_offset;
+	  new_trapezoids[i].x11 = trapezoids[i].x11 - x_offset;
+	  new_trapezoids[i].x21 = trapezoids[i].x21 - x_offset;
+	  new_trapezoids[i].y2 = trapezoids[i].y2 - y_offset;
+	  new_trapezoids[i].x12 = trapezoids[i].x12 - x_offset;
+	  new_trapezoids[i].x22 = trapezoids[i].x22 - x_offset;
+	}
+
+      trapezoids = new_trapezoids;
+    }
+
+  if (private->paint_stack)
+    {
+      GdkWindowPaint *paint = private->paint_stack->data;
+      gdk_draw_trapezoids (paint->pixmap, gc, trapezoids, n_trapezoids);
+    }
+  else
+    gdk_draw_trapezoids (private->impl, gc, trapezoids, n_trapezoids);
+  
+  if (new_trapezoids)
+    g_free (new_trapezoids);
+
+  RESTORE_GC (gc);
+}
+
 static void
 gdk_window_real_get_size (GdkDrawable *drawable,
                           gint *width,
@@ -1984,13 +2104,57 @@ gdk_window_copy_to_image (GdkDrawable     *drawable,
 				     width, height);
 }
 
+static cairo_surface_t *
+gdk_window_ref_cairo_surface (GdkDrawable *drawable)
+{
+  GdkWindowObject *private = (GdkWindowObject*) drawable;
+  cairo_surface_t *surface;
+  gint x_offset, y_offset;
+  
+  gdk_window_get_offsets (GDK_WINDOW (drawable), &x_offset, &y_offset);
+
+  if (private->paint_stack)
+    {
+      GdkWindowPaint *paint = private->paint_stack->data;
+
+      surface = paint->surface;
+      cairo_surface_reference (surface);
+    }
+  else
+    surface = _gdk_drawable_ref_cairo_surface (private->impl);
+
+  return surface;
+}
+
 /* Code for dirty-region queueing
  */
-
 static GSList *update_windows = NULL;
 static guint update_idle = 0;
 static gboolean debug_updates = FALSE;
 
+static gboolean
+gdk_window_update_idle (gpointer data)
+{
+  GDK_THREADS_ENTER ();
+  gdk_window_process_all_updates ();
+  GDK_THREADS_LEAVE ();
+  
+  return FALSE;
+}
+
+static void
+gdk_window_schedule_update (GdkWindow *window)
+{
+  if (window && GDK_WINDOW_OBJECT (window)->update_freeze_count)
+    return;
+
+  if (!update_idle)
+    {
+      update_idle = g_idle_add_full (GDK_PRIORITY_REDRAW,
+				     gdk_window_update_idle, NULL, NULL);
+    }
+}
+
 static void
 gdk_window_process_updates_internal (GdkWindow *window)
 {
@@ -2006,10 +2170,8 @@ gdk_window_process_updates_internal (GdkWindow *window)
       GdkRegion *update_area = private->update_area;
       private->update_area = NULL;
       
-      if (_gdk_event_func && gdk_window_is_viewable (window) &&
-	  (private->event_mask & GDK_EXPOSURE_MASK))
+      if (_gdk_event_func && gdk_window_is_viewable (window))
 	{
-	  GdkEvent event;
 	  GdkRectangle window_rect;
 	  GdkRegion *expose_region;
 	  GdkRegion *window_region;
@@ -2022,39 +2184,44 @@ gdk_window_process_updates_internal (GdkWindow *window)
               g_usleep (70000);
             }
           
-          gdk_drawable_get_size (GDK_DRAWABLE (private), &width, &height);
-          
-	  window_rect.x = 0;
-	  window_rect.y = 0;
-	  window_rect.width = width;
-	  window_rect.height = height;
-
 	  save_region = _gdk_windowing_window_queue_antiexpose (window, update_area);
-      
-	  event.expose.type = GDK_EXPOSE;
-	  event.expose.window = g_object_ref (window);
-	  event.expose.count = 0;
 
 	  if (save_region)
 	    expose_region = gdk_region_copy (update_area);
 	  else
 	    expose_region = update_area;
+	  
+          gdk_drawable_get_size (GDK_DRAWABLE (private), &width, &height);
+
+	  window_rect.x = 0;
+	  window_rect.y = 0;
+	  window_rect.width = width;
+	  window_rect.height = height;
+
 	  window_region = gdk_region_rectangle (&window_rect);
 	  gdk_region_intersect (expose_region,
 				window_region);
 	  gdk_region_destroy (window_region);
 	  
-	  event.expose.region = expose_region;
-	  gdk_region_get_clipbox (expose_region, &event.expose.area);
-	  
-	  if (!gdk_region_empty (expose_region))
+	  if (!gdk_region_empty (expose_region) &&
+	      (private->event_mask & GDK_EXPOSURE_MASK))
 	    {
+	      GdkEvent event;
+	      
+	      event.expose.type = GDK_EXPOSE;
+	      event.expose.window = g_object_ref (window);
+	      event.expose.send_event = FALSE;
+	      event.expose.count = 0;
+	      event.expose.region = expose_region;
+	      gdk_region_get_clipbox (expose_region, &event.expose.area);
+	      
 	      (*_gdk_event_func) (&event, _gdk_event_data);
+	      
+	      g_object_unref (window);
 	    }
 
 	  if (expose_region != update_area)
 	    gdk_region_destroy (expose_region);
-	  g_object_unref (window);
 	}
       if (!save_region)
 	gdk_region_destroy (update_area);
@@ -2096,7 +2263,13 @@ gdk_window_process_all_updates (void)
   
   while (tmp_list)
     {
-      gdk_window_process_updates_internal (tmp_list->data);
+      GdkWindowObject *private = (GdkWindowObject *)tmp_list->data;
+      
+      if (private->update_freeze_count)
+	update_windows = g_slist_prepend (update_windows, private);
+      else
+	gdk_window_process_updates_internal (tmp_list->data);
+      
       g_object_unref (tmp_list->data);
       tmp_list = tmp_list->next;
     }
@@ -2106,16 +2279,6 @@ gdk_window_process_all_updates (void)
   flush_all_displays ();
 }
 
-static gboolean
-gdk_window_update_idle (gpointer data)
-{
-  GDK_THREADS_ENTER ();
-  gdk_window_process_all_updates ();
-  GDK_THREADS_LEAVE ();
-  
-  return FALSE;
-}
-
 /**
  * gdk_window_process_updates:
  * @window: a #GdkWindow
@@ -2140,7 +2303,7 @@ gdk_window_process_updates (GdkWindow *window,
   g_return_if_fail (window != NULL);
   g_return_if_fail (GDK_IS_WINDOW (window));
   
-  if (private->update_area)
+  if (private->update_area && !private->update_freeze_count)
     {      
       gdk_window_process_updates_internal (window);
       update_windows = g_slist_remove (update_windows, window);
@@ -2258,6 +2421,7 @@ gdk_window_invalidate_maybe_recurse (GdkWindow *window,
 {
   GdkWindowObject *private = (GdkWindowObject *)window;
   GdkRegion *visible_region;
+  GList *tmp_list;
 
   g_return_if_fail (window != NULL);
   g_return_if_fail (GDK_IS_WINDOW (window));
@@ -2271,6 +2435,45 @@ gdk_window_invalidate_maybe_recurse (GdkWindow *window,
   visible_region = gdk_drawable_get_visible_region (window);
   gdk_region_intersect (visible_region, region);
 
+  tmp_list = private->children;
+  while (tmp_list)
+    {
+      GdkWindowObject *child = tmp_list->data;
+      
+      if (!child->input_only)
+	{
+	  GdkRegion *child_region;
+	  GdkRectangle child_rect;
+	  
+	  gdk_window_get_position ((GdkWindow *)child,
+				   &child_rect.x, &child_rect.y);
+	  gdk_drawable_get_size ((GdkDrawable *)child,
+				 &child_rect.width, &child_rect.height);
+
+	  child_region = gdk_region_rectangle (&child_rect);
+	  
+	  /* remove child area from the invalid area of the parent */
+	  if (GDK_WINDOW_IS_MAPPED (child))
+	    gdk_region_subtract (visible_region, child_region);
+	  
+	  if (child_func && (*child_func) ((GdkWindow *)child, user_data))
+	    {
+	      gdk_region_offset (region, - child_rect.x, - child_rect.y);
+	      gdk_region_offset (child_region, - child_rect.x, - child_rect.y);
+	      gdk_region_intersect (child_region, region);
+	      
+	      gdk_window_invalidate_maybe_recurse ((GdkWindow *)child,
+						   child_region, child_func, user_data);
+	      
+	      gdk_region_offset (region, child_rect.x, child_rect.y);
+	    }
+
+	  gdk_region_destroy (child_region);
+	}
+
+      tmp_list = tmp_list->next;
+    }
+  
   if (!gdk_region_empty (visible_region))
     {
       if (debug_updates)
@@ -2285,37 +2488,7 @@ gdk_window_invalidate_maybe_recurse (GdkWindow *window,
 	  update_windows = g_slist_prepend (update_windows, window);
 	  private->update_area = gdk_region_copy (visible_region);
 	  
-	  if (!private->update_freeze_count && !update_idle)
-	    update_idle = g_idle_add_full (GDK_PRIORITY_REDRAW,
-					   gdk_window_update_idle, NULL, NULL);
-	}
-      
-      if (child_func)
-	{
-	  GList *tmp_list;
-	  
-	  tmp_list = private->children;
-	  while (tmp_list)
-	    {
-	      GdkWindowObject *child = tmp_list->data;
-	      tmp_list = tmp_list->next;
-
-	      if (!child->input_only && (*child_func) ((GdkWindow *)child, user_data))
-		{
-		  GdkRegion *child_region;
-		  gint x, y;
-
-		  gdk_window_get_position ((GdkWindow *)child, &x, &y);
-
-		  /* This copy could be saved with a little more complexity */
-		  child_region = gdk_region_copy (visible_region);
-		  gdk_region_offset (child_region, - x, - y);
-		  
-		  gdk_window_invalidate_maybe_recurse ((GdkWindow *)child, child_region, child_func, user_data);
-		  
-		  gdk_region_destroy (child_region);
-		}
-	    }
+	  gdk_window_schedule_update (window);
 	}
     }
   
@@ -2459,10 +2632,8 @@ gdk_window_thaw_updates (GdkWindow *window)
   g_return_if_fail (GDK_IS_WINDOW (window));
   g_return_if_fail (private->update_freeze_count > 0);
 
-  private->update_freeze_count--;
-  if (!private->update_freeze_count && private->update_area && !update_idle)
-    update_idle = g_idle_add_full (GDK_PRIORITY_REDRAW,
-				   gdk_window_update_idle, NULL, NULL);
+  if (--private->update_freeze_count == 0)
+    gdk_window_schedule_update (window);
 }
 
 /**
@@ -2736,3 +2907,5 @@ gdk_window_foreign_new (GdkNativeWindow anid)
   return gdk_window_foreign_new_for_display (gdk_display_get_default (), anid);
 }
 
+#define __GDK_WINDOW_C__
+#include "gdkaliasdef.c"
