@@ -20,12 +20,12 @@
  */
 
 #include <config.h>
-#include "gtkalias.h"
 #include "gtkfilesystem.h"
 #include "gtkfilesystemwin32.h"
 #include "gtkintl.h"
 #include "gtkstock.h"
 #include "gtkiconfactory.h"
+#include "gtkalias.h"
 
 #include <glib/gstdio.h>
 
@@ -38,12 +38,12 @@
 #ifdef G_OS_WIN32
 #define WIN32_LEAN_AND_MEAN
 #define STRICT
-#include <windows.h>
+#include "gdk/win32/gdkwin32.h"
 #undef STRICT
 #include <shlobj.h>
 #include <shellapi.h>
 #else
-#error "The implementation is win32 only."
+#error "The implementation is Win32 only."
 #endif /* G_OS_WIN32 */
 
 typedef struct _GtkFileSystemWin32Class GtkFileSystemWin32Class;
@@ -606,26 +606,54 @@ gtk_file_system_win32_volume_get_display_name (GtkFileSystem       *file_system,
 
   g_return_val_if_fail (volume->drive != NULL, NULL);
 
-  if ((filename_is_drive_root (volume->drive) && volume->drive[0] >= 'C') ||
+  if (filename_is_drive_root (volume->drive) &&
+      volume->drive_type == DRIVE_REMOTE)
+    real_display_name = g_strdup_printf (_("Network Drive (%s)"), volume->drive);
+  else if ((filename_is_drive_root (volume->drive) && volume->drive[0] >= 'C') ||
       volume->drive_type != DRIVE_REMOVABLE)
     {
-      gunichar2 *wdrive = g_utf8_to_utf16 (volume->drive, -1, NULL, NULL, NULL);
-      gunichar2 wname[80];
-      if (GetVolumeInformationW (wdrive,
-				 wname, G_N_ELEMENTS(wname), 
-				 NULL, /* serial number */
-				 NULL, /* max. component length */
-				 NULL, /* fs flags */
-				 NULL, 0) /* fs type like FAT, NTFS */ &&
-	  wname[0])
+      gchar *name = NULL;
+      if (G_WIN32_HAVE_WIDECHAR_API ())
 	{
-	  gchar *name = g_utf16_to_utf8 (wname, -1, NULL, NULL, NULL);
-	  real_display_name = g_strconcat (name, " (", volume->drive, ")", NULL);
+	  gunichar2 *wdrive = g_utf8_to_utf16 (volume->drive, -1, NULL, NULL, NULL);
+	  gunichar2 wname[80];
+	  if (GetVolumeInformationW (wdrive,
+				     wname, G_N_ELEMENTS(wname), 
+				     NULL, /* serial number */
+				     NULL, /* max. component length */
+				     NULL, /* fs flags */
+				     NULL, 0) /* fs type like FAT, NTFS */ &&
+	      wname[0])
+	    {
+	      name = g_utf16_to_utf8 (wname, -1, NULL, NULL, NULL);
+	    }
+	  g_free (wdrive);
+	}
+      else
+        {
+          gchar *cpdrive = g_locale_from_utf8 (volume->drive, -1, NULL, NULL, NULL);
+          gchar cpname[80];
+          if (GetVolumeInformationA (cpdrive,
+				     cpname, G_N_ELEMENTS(cpname), 
+				     NULL, /* serial number */
+				     NULL, /* max. component length */
+				     NULL, /* fs flags */
+				     NULL, 0) /* fs type like FAT, NTFS */ &&
+	      cpname[0])
+            {
+              name = g_locale_to_utf8 (cpname, -1, NULL, NULL, NULL);
+            }
+          g_free (cpdrive);
+        }
+      if (name != NULL)
+        {
+	  real_display_name = g_strdup_printf (_("%s (%s)"), name, volume->drive);
 	  g_free (name);
 	}
       else
-	real_display_name = g_strdup (volume->drive);
-      g_free (wdrive);
+	{
+	  real_display_name = g_strdup (volume->drive);
+	}
     }
   else
     real_display_name = g_strdup (volume->drive);
@@ -738,7 +766,7 @@ canonicalize_filename (gchar *filename)
   printf("canonicalize_filename: %s ", filename);
 #endif
 
-  past_root = g_path_skip_root (filename);
+  past_root = (gchar *) g_path_skip_root (filename);
 
   q = p = past_root;
 
@@ -1014,7 +1042,6 @@ extract_icon (const char* filename)
 {
   GdkPixbuf *pixbuf = NULL;
   HICON hicon;
-  ICONINFO ii;
   
   if (!filename || !filename[0])
     return NULL;
@@ -1097,79 +1124,7 @@ extract_icon (const char* filename)
     }
 #endif
   
-  if (GetIconInfo (hicon, &ii))
-    {
-      struct
-      {
-	BITMAPINFOHEADER bi;
-	RGBQUAD colors[2];
-      } bmi;
-      HDC hdc;
-      
-      memset (&bmi, 0, sizeof (bmi));
-      bmi.bi.biSize = sizeof (bmi.bi);
-      hdc = CreateCompatibleDC (NULL);
-      
-      if (GetDIBits (hdc, ii.hbmColor, 0, 1, NULL, (BITMAPINFO *)&bmi, DIB_RGB_COLORS))
-	{
-	  gchar *pixels, *bits;
-	  gint rowstride, x, y, w = bmi.bi.biWidth, h = bmi.bi.biHeight;
-	  gboolean no_alpha;
-	  
-	  bmi.bi.biBitCount = 32;
-	  bmi.bi.biCompression = BI_RGB;
-	  bmi.bi.biHeight = -h;
-	  pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8, w, h);
-	  bits = g_malloc0 (4 * w * h);
-	  
-	  /* color data */
-	  if (!GetDIBits (hdc, ii.hbmColor, 0, h, bits, (BITMAPINFO *)&bmi, DIB_RGB_COLORS))
-	    g_warning (G_STRLOC ": Failed to get dibits");
-	  
-	  pixels = gdk_pixbuf_get_pixels (pixbuf);
-	  rowstride = gdk_pixbuf_get_rowstride (pixbuf);
-	  no_alpha = TRUE;
-	  for (y = 0; y < h; y++)
-	    {
-	      for (x = 0; x < w; x++)
-		{
-		  pixels[2] = bits[(x+y*w) * 4];
-		  pixels[1] = bits[(x+y*w) * 4 + 1];
-		  pixels[0] = bits[(x+y*w) * 4 + 2];
-		  pixels[3] = bits[(x+y*w) * 4 + 3];
-		  if (no_alpha && pixels[3] > 0) no_alpha = FALSE;
-		  pixels += 4;
-		}
-	      pixels += (w * 4 - rowstride);
-	    }
-	  /* mask */
-	  if (no_alpha) {
-	    if (!GetDIBits (hdc, ii.hbmMask, 0, h, bits, (BITMAPINFO *)&bmi, DIB_RGB_COLORS))
-	      g_warning (G_STRLOC ": Failed to get dibits");
-	    pixels = gdk_pixbuf_get_pixels (pixbuf);
-	    for (y = 0; y < h; y++)
-	      {
-		for (x = 0; x < w; x++)
-		  {
-		    pixels[3] = 255 - bits[(x + y * w) * 4];
-		    pixels += 4;
-		  }
-		pixels += (w * 4 - rowstride);
-	      }
-	    
-	    /* release temporary resources */
-	    g_free (bits);
-	    if (!DeleteObject (ii.hbmColor) || !DeleteObject (ii.hbmMask))
-	      g_warning (G_STRLOC ": Leaking Icon Bitmaps ?");
-	  }
-	}
-      else
-	g_warning (G_STRLOC ": GetDIBits () failed, %s", g_win32_error_message (GetLastError ()));
-      
-      DeleteDC (hdc);
-    }
-  else
-    g_warning (G_STRLOC ": GetIconInfo failed: %s\n", g_win32_error_message (GetLastError ())); 
+  pixbuf = gdk_win32_icon_to_pixbuf_libgtk_only (hicon);
   
   if (!DestroyIcon (hicon))
     g_warning (G_STRLOC ": DestroyIcon failed: %s\n", g_win32_error_message (GetLastError ()));
@@ -1284,10 +1239,12 @@ gtk_file_system_win32_render_icon (GtkFileSystem     *file_system,
     }
 
   if (!icon_set)
-    if (g_file_test (filename, G_FILE_TEST_IS_EXECUTABLE))
-	icon_set = gtk_style_lookup_icon_set (widget->style, GTK_STOCK_EXECUTE);
-    else
-      icon_set = gtk_style_lookup_icon_set (widget->style, GTK_STOCK_FILE);
+    {
+      if (g_file_test (filename, G_FILE_TEST_IS_EXECUTABLE))
+        icon_set = gtk_style_lookup_icon_set (widget->style, GTK_STOCK_EXECUTE);
+      else
+        icon_set = gtk_style_lookup_icon_set (widget->style, GTK_STOCK_FILE);
+    }
 
   // FIXME : I'd like to get from pixel_size (=20) back to
   // icon size, which is an index, but there appears to be no way ?
@@ -1708,3 +1665,6 @@ _gtk_file_system_win32_path_compare (const gchar *path1,
 
   return retval;
 }
+
+#define __GTK_FILE_SYSTEM_WIN32_C__
+#include "gtkaliasdef.c"
